using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using MagLibTest; //protected Maglib DLL 
using PdfLibTest; //protected PdfLib DLL
using System.IO.Ports;
using DLLParsingTest.Properties;//needed to save app.config
using BuffMgr;//needed to build a scan or swipe


/*****************************************************************/
/* Tokenworks PDF417 and Magnetic Stripe Parsing Library         */
/*  Copyright 2015 TokenWorks Inc.   All rights reserved.       */
/* Redistribution and use of this code in source and/or binary   */
/* forms, with or without modification, are permitted provided   */
/* that: (1) all copies of the source code retain the above      */
/* unmodified copyright notice and this entire unmodified        */
/* section of text, (2) You or Your organization owns a valid    */
/* Developer License to this product from TokenWorks Inc.        */
/* and, (3) when any portion of this code is bundled in any      */
/* form with an application, a valid notice must be provided     */
/* within the user documentation, start-up screen or in the      */
/* help-about section of the application that specifies          */
/* TokenWorks as the provider of the Software bundled            */
/* with the application.                                         */
/*****************************************************************/


// SDK version 2.1 - Desktop ID/DL Parsing library demo
// SDK Version 2.8 - Add Scanner interface and Bank Card fields to app.
// SDK Version 3.5 - Add Library Interface
namespace DLLParsingTest
{
    public partial class frmMain : Form
    {
        MagLibTest.Magstripe mc;
        PdfLibTest.Barcode bc;
        #region Local Variables
       public static SerialPort comport = new SerialPort();
        string stateP_DL;
        string cityP_DL;
        string addrP_DL;
        string addr2_DL;
        string firstnameP_DL;
        string lastnameP_DL;
        string titleP_DL;
        string midnameP_DL;
        string dlP_DL;
        string expP_DL;
        string dobyearP_DL;
        string dobMMDDP_DL;
        string zipP_DL;
        string HeightP_DL;
        string EyeColorP_DL;
        string SexP_DL;
        string ClassP_DL;
        string RestrictionsP_DL;
        string EndorsementsP_DL;
        string WeightP_DL;
        string HairP_DL;



        byte[] MagBuff = new byte[315]; //Magstrip reader buffer size limit
        byte[] buff = new byte[2000]; // PDF417 barcode reader buffer size limit
        byte[] Rawbuff = new byte[2000]; // RawData buffer 

        string displaystring;
        int ss1 = 0; //start sentinel 1
        int ss2 = 1; //start sentinel 2
        int ss3 = 2; //start sentinel 3
        int MBD = 3; //Magstripe Barcode Designator

        DateTime d1 = DateTime.Now;
        #endregion


        public frmMain()
        {
            InitializeComponent();

            // When data is recieved through the port, call this method
            comport.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);

            //Initialize the time stamp
            ReaderBuffMgr.GetTimeStamp();

        }

        #region Local Functions

        /// <summary> Convert a string of hex digits (ex: E4 CA B2) to a byte array. </summary>
        /// <param name="s"> The string containing the hex digits (with or without spaces). </param>
        /// <returns> Returns an array of bytes. </returns>
        public static byte[] HexStringToByteArray(string s)
        {
            s = s.Replace(" ", "");
            byte[] buffer = new byte[s.Length / 2];
            for (int i = 0; i < s.Length; i += 2)
                buffer[i / 2] = (byte)Convert.ToByte(s.Substring(i, 2), 16);
            return buffer;
        }

        /// <summary> Converts an array of bytes into a formatted string of hex digits (ex: E4 CA B2)</summary>
        /// <param name="data"> The array of bytes to be translated into a string of hex digits. </param>
        /// <returns> Returns a well formatted string of hex digits with spacing. </returns>
        public string ByteArrayToHexString(byte[] data)
        {
            StringBuilder sb = new StringBuilder(data.Length * 3);
            foreach (byte b in data)
                sb.Append(Convert.ToString(b, 16).PadLeft(2, '0').PadRight(3, ' '));
            return sb.ToString().ToUpper();
        }

        #endregion



        #region Events

        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {

            // Obtain the number of bytes waiting in the port's buffer
            int bytes = comport.BytesToRead;

            // Create a byte array buffer to hold the incoming data
            byte[] buffer = new byte[bytes];

            // Read the data from the port and store it in our buffer
            comport.Read(buffer, 0, bytes);
            ReaderBuffMgr.DataBuilder(buffer, Environment.TickCount);
            d1 = DateTime.Now; //update timestamp

            //test for a complete scan before calling pdf417 parsing
            if ((ReaderBuffMgr.RawBytes[ss1] == 0x01) && (ReaderBuffMgr.RawBytes[ss2] == 0x02) && (ReaderBuffMgr.RawBytes[ss3] == 0x5D) && (ReaderBuffMgr.RawBytes[MBD] == 0x4C) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
            {
                Array.Resize(ref buff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                Array.Copy(ReaderBuffMgr.RawBytes, 4, buff, 0, ReaderBuffMgr.PrevLng); //you Must strip off the 4 pre-fix bytes inserte by the scanner

                ReaderBuffMgr.PrevLng = 0; //reset conters
                ReaderBuffMgr.count = 0;  //for next scan

                try
                {
                    this.Invoke(new EventHandler(SendPDF));  // always call the DLLs with a try/catch
                }
                catch
                {
                    MessageBox.Show("Invalid Barcode");
                }


                displaystring = PdfLibTest.Barcode.PDF_String;
                this.Invoke(new EventHandler(SetText));
            }

            //test for magstripe read from scanner
            if ((ReaderBuffMgr.RawBytes[ss1] == 01) && (ReaderBuffMgr.RawBytes[ss2] == 02) && (ReaderBuffMgr.RawBytes[ss3] == 0x5D) && (ReaderBuffMgr.RawBytes[MBD] == 0x4D) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
            {
                Array.Resize(ref MagBuff, ReaderBuffMgr.PrevLng); //You MUST resize the global to pass only current data
                ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng] = 0;//insert null to create printable string
                Array.Copy(ReaderBuffMgr.RawBytes, 4, MagBuff, 0, ReaderBuffMgr.PrevLng);//you Must strip off the 4 pre-fix bytes inserte by the scanner

                ReaderBuffMgr.PrevLng = 0; //reset counters
                ReaderBuffMgr.count = 0; //for next swipe

                 try
                {
                    this.Invoke(new EventHandler(SendMAG)); // always call the DLLs with a try/catch
                }
                catch
                {
                    MessageBox.Show("Invalid Magstripe");
                }


                displaystring = MagLibTest.Magstripe.track1 + MagLibTest.Magstripe.track2 + MagLibTest.Magstripe.track3;
                this.Invoke(new EventHandler(SetText));
            }

            //this will catch bad barcode reads
            if ((ReaderBuffMgr.RawBytes[ss1] == 01) && (ReaderBuffMgr.RawBytes[ss2] == 02) && (ReaderBuffMgr.RawBytes[ss3] == 0x4E) && (ReaderBuffMgr.RawBytes[MBD] == 0x52) && (ReaderBuffMgr.RawBytes[ReaderBuffMgr.PrevLng - 1] == 0x04))
            {
                ReaderBuffMgr.PrevLng = 0;
                ReaderBuffMgr.count = 0;

                displaystring = "\r\n Bad Barcode Read, Try Again";
                this.Invoke(new EventHandler(SetText));

            }

            //this catches the ACK from the Ping command
            if ((ReaderBuffMgr.RawBytes[0] == 0x06) && (ReaderBuffMgr.PrevLng == 0x01))
            {
                ReaderBuffMgr.PrevLng = 0;
                ReaderBuffMgr.count = 0;
                // Show the user the incoming data in hex format
                displaystring = "OK!";
                this.Invoke(new EventHandler(SetText));
            }
        }


        private void SetText(object o, EventArgs e)
        {
            txtOutput.Text = displaystring;
        }


        private void btnMagTest_Click(object sender, EventArgs e)
        {

            //Magstripe_Scan is a simulated buffer of magnetic card data, in this case all pre-fix,postfix data has already been removed.
            byte[] Magstripe_Scan = HexStringToByteArray("25 4E 59 41 4C 42 41 4E 59 5E 54 4F 4B 45 4E 57 4F 52 4B 53 24 4A 4F 45 34 24 54 5E 38 33 34 20 43 4F 4E 43 48 20 53 48 45 4C 4C 20 52 44 20 20 20 20 23 39 30 31 5E 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 3F 3B 36 33 36 30 31 30 30 33 32 34 31 36 38 33 36 33 31 33 37 3D 30 37 30 32 31 39 38 34 39 39 30 34 30 3D 3F 25 21 20 31 32 33 34 34 30 30 30 30 20 20 45 20 42 20 20 20 20 20 20 20 20 20 20 20 20 20 31 35 30 36 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 4E 2F 26 2B 47 47 30 30 30 30 30 3F");
            Array.Resize(ref MagBuff, Magstripe_Scan.Length); //You MUST resize the data, the buffer sent must contain only card data, no preceding bytes generated by a scanner, and no trailing garbage data. Normally the length is created from the SerialPort.BytesToRead property.
            Array.Copy(Magstripe_Scan, 0, MagBuff, 0, Magstripe_Scan.Length); // copy the scan from your SerialDataReceivedEventHandler into the global MagBuff array

            try
            {
                this.Invoke(new EventHandler(SendMAG)); // always call the DLLs with a try/catch
            }
            catch
            {
                MessageBox.Show("Invalid Magstripe");
            }
            displaystring = MagLibTest.Magstripe.track1 + MagLibTest.Magstripe.track2 + MagLibTest.Magstripe.track3; // display the ASCII version of the input data
            this.Invoke(new EventHandler(SetText)); 
            


        }

        private void btnPdfTest_Click(object sender, EventArgs e)
        {

            //BarCode_Scan is a simulated buffer of PDF417 Barcode card data, in this case all pre-fix,postfix data has already been removed.
            byte[] BarCode_Scan = HexStringToByteArray("40 0A 1C 0D 41 41 4D 56 41 33 36 30 30 31 30 30 31 44 4C 30 30 31 32 33 34 35 36 44 4C 0A 44 41 41 54 4F 4B 45 4E 57 4F 52 4B 53 40 54 4F 4D 40 54 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 0A 44 41 47 34 34 20 45 20 39 39 54 48 20 53 54 20 41 50 54 20 39 30 31 0A 44 41 49 4E 45 57 20 59 4F 52 4B 20 20 20 20 20 20 20 0A 44 41 4A 4E 59 0A 44 41 4B 31 30 31 32 38 20 20 20 20 20 20 0A 44 41 51 33 36 32 38 33 32 35 31 32 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 0A 20 44 42 42 31 39 38 37 30 38 30 32 0A 44 42 44 32 30 30 30 31 30 30 34 0A 44 42 41 32 30 31 33 30 35 30 37 0A 44 41 52 44 4D 20 20 0A 44 41 53 20 20 20 20 20 20 20 20 20 20 0A 44 41 54 20 20 20 20 20 20 0A 44 42 43 4D 0A 44 41 55 36 30 32 0A 44 41 59 42 52 20 0A 44 42 4B 20 20 20 20 20 20 20 20 20 0A 44 42 48 20 20 20 20 20 20 20 20 20 20 0D"); 
            Array.Resize(ref MagBuff, BarCode_Scan.Length); //You MUST resize the data, the buffer sent must contain only card data, no preceding bytes generated by a scanner, and no trailing garbage data. Normally the length is created from the SerialPort.BytesToRead property.

            Array.Copy(BarCode_Scan, buff, BarCode_Scan.Length);// copy input to global buffer

            try
            {
                this.Invoke(new EventHandler(SendPDF));  // always call the DLLs with a try/catch
            }
            catch
            {
                MessageBox.Show("Invalid Barcode");
            }
            displaystring = PdfLibTest.Barcode.PDF_String; // display the ASCII version of the input data
            this.Invoke(new EventHandler(SetText));


        }



        private void btnClear_Click(object sender, EventArgs e)
        {
            txtFirst.Text = "";
            txtLast.Text = "";
            txtMid.Text = "";
            txtTitle.Text = "";
            txtID.Text = "";
            txtState.Text = "";
            txtCity.Text = "";
            txtAddress.Text = "";
            txtEXP.Text = "";
            txtZip.Text = "";
            txtClass.Text = "";
            txtRestriction.Text = "";
            txtEndorsement.Text = "";
            txtSEX.Text = "";
            txtHeight.Text = "";
            txtWeight.Text = "";
            txtHair.Text = "";
            txtEye.Text = "";
            txtDobMMDD.Text = "";
            txtDobYear.Text = "";
            txtCountry.Text = ""; 
            txtRace.Text = ""; 
            txtDBD.Text = ""; 
            txtAddr2.Text = ""; 
            txtRetMag.Text = "";
            txtRetPDF.Text = "";
            txtVersion.Text = "";

            displaystring = "";
            this.Invoke(new EventHandler(SetText));
        }

        private void SendMAG(object sender, EventArgs e)
        {
 
            int ret = 0;
            try
            {
                mc = new MagLibTest.Magstripe();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error loading MagLib");
                return;

            }


            ret = mc.ParseDL(MagBuff); // Parse the buffer



                stateP_DL = Magstripe.stateP_DL;
                cityP_DL = Magstripe.cityP_DL;
                addrP_DL = Magstripe.addrP_DL;
                firstnameP_DL = Magstripe.firstnameP_DL;
                lastnameP_DL = Magstripe.lastnameP_DL;
                midnameP_DL = Magstripe.midnameP_DL;
                titleP_DL = Magstripe.titleP_DL;
                dlP_DL = Magstripe.dlP_DL;
                expP_DL = Magstripe.expP_DL;
                dobyearP_DL = Magstripe.dobyearP_DL;
                dobMMDDP_DL = Magstripe.dobMMDDP_DL;
                zipP_DL = Magstripe.zipP_DL;
                ClassP_DL = Magstripe.ClassP_DL;
                RestrictionsP_DL = Magstripe.RestrictionsP_DL;
                EndorsementsP_DL = Magstripe.EndorsementsP_DL;
                SexP_DL = Magstripe.SexP_DL;
                HeightP_DL = Magstripe.HeightP_DL;
                WeightP_DL = Magstripe.WeightP_DL;
                HairP_DL = Magstripe.HairP_DL;
                EyeColorP_DL = Magstripe.EyeP_DL;


                txtFirst.Text = firstnameP_DL;
                txtLast.Text = lastnameP_DL;
                txtMid.Text = midnameP_DL;
                txtTitle.Text = titleP_DL;
                txtID.Text = dlP_DL;
                txtState.Text = stateP_DL;
                txtCity.Text = cityP_DL;
                txtAddress.Text = addrP_DL;
                txtEXP.Text = expP_DL;
                txtZip.Text = zipP_DL;
                txtClass.Text = ClassP_DL;
                txtRestriction.Text = RestrictionsP_DL;
                txtEndorsement.Text = EndorsementsP_DL;
                txtSEX.Text = SexP_DL;
                txtHeight.Text = HeightP_DL;
                txtWeight.Text = WeightP_DL;
                txtHair.Text = HairP_DL;
                txtEye.Text = EyeColorP_DL;
                txtDobMMDD.Text = dobMMDDP_DL;
                txtDobYear.Text = dobyearP_DL;
                txtRetMag.Text = ret.ToString();
                txtVersion.Text = Magstripe.Version;
//These fields are available on barcode only
                txtCountry.Text = ""; //n/a
                txtRace.Text = ""; // n/a
                txtDBD.Text = ""; // n/a
                txtAddr2.Text = ""; // n/a
                //display return code
                

        }

        private void SendPDF(object sender, EventArgs e)
        {

            int i = -1;
            try
            {
                bc = new PdfLibTest.Barcode();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error loading PdfLib");
                return;

            }

            i = bc.Parse417(buff); // Parse the buffer
            
            stateP_DL = Barcode.State_DL;
            cityP_DL = Barcode.City_DL;
            addrP_DL = Barcode.Addr_DL;
            firstnameP_DL = Barcode.First_DL;
            lastnameP_DL = Barcode.Last_DL;
            midnameP_DL = Barcode.Mid_DL;
            titleP_DL = Barcode.Title_DL;
            dlP_DL = Barcode.Dl_DL;
            expP_DL = Barcode.Exp_DL;
            dobyearP_DL = Barcode.DobYear_DL;
            dobMMDDP_DL = Barcode.DobMMDD_DL;
            zipP_DL = Barcode.Zip_DL;
            HeightP_DL = Barcode.Height_DL;
            EyeColorP_DL = Barcode.Eye_DL;
            SexP_DL = Barcode.Sex_DL;
            ClassP_DL = Barcode.Class_DL;
            RestrictionsP_DL = Barcode.Rest_DL;
            EndorsementsP_DL = Barcode.Endr_DL;
            WeightP_DL = Barcode.Wt_DL;
            HairP_DL = Barcode.Hair_DL;
            addr2_DL = Barcode.Addr2_DL;
            
                        //Loading Gui Object from local app variables
                        txtFirst.Text = firstnameP_DL;
                        txtLast.Text = lastnameP_DL;
                        txtMid.Text = midnameP_DL;
                        txtAddress.Text = addrP_DL;
                        txtCity.Text = cityP_DL;
                        txtState.Text = stateP_DL;
                        txtZip.Text = zipP_DL;
                        txtID.Text = dlP_DL;
                        txtSEX.Text = SexP_DL;
                        txtEXP.Text = expP_DL;
                        txtTitle.Text = titleP_DL;
                        txtEye.Text = EyeColorP_DL;
                        txtHeight.Text = HeightP_DL;
                        txtDobMMDD.Text = dobMMDDP_DL;
                        txtDobYear.Text = dobyearP_DL;
                        txtClass.Text = ClassP_DL;
                        txtRestriction.Text = RestrictionsP_DL;
                        txtEndorsement.Text = EndorsementsP_DL;
                        txtWeight.Text = WeightP_DL;
                        txtHair.Text = HairP_DL;
                        txtRetPDF.Text = i.ToString();

                        // loading directly from library

                        //These fields are available on barcode only
                        txtAddr2.Text = Barcode.Addr2_DL;
                        txtCountry.Text = Barcode.Country_DL;
                        txtRace.Text = Barcode.Race_DL;
                        txtDBD.Text = Barcode.DocIssDate_DL;
                        txtVersion.Text = Barcode.Version;
             


        }




        private void btnPort_Click(object sender, EventArgs e)
        {
            (new frmSettings()).ShowDialog();
        }

        private void btnOpenPort_Click(object sender, EventArgs e)
        {
            // If the port is open, close it.
            if (comport.IsOpen) comport.Close();
            else
            {
                comport.BaudRate = Settings.Default.BaudRate;
                comport.DataBits = Settings.Default.DataBits;
                comport.StopBits = Settings.Default.StopBits;
                comport.Parity = Settings.Default.Parity;
                comport.PortName = Settings.Default.PortName;
                // Open the port
                try
                {
                    comport.Open();
                }
                catch (Exception px)
                {
                    MessageBox.Show(px.Message, "Port already in use");
                }


            }
            if (comport.IsOpen)
            {
                btnOpenPort.Text = "&Close Port";
                btnMagTest.Enabled = false;
                btnPdfTest.Enabled = false;

            }

            else
            {
                btnOpenPort.Text = "&Open Port";
                btnMagTest.Enabled = true;
                btnPdfTest.Enabled = true;

            }
        }

        private void btnTest2_Click(object sender, EventArgs e)
        {
            if (comport.IsOpen)
            {
                //this commands pings the port to see if the reader is connected
           
                byte[] data1 = HexStringToByteArray("01 3C 54 58 50 49 4E 47 3A 3E 04"); //
                // Send the binary data out the port
                comport.Write(data1, 0, data1.Length);
            }

        }

        private void btnProgram_Click(object sender, EventArgs e)
        {
            if (comport.IsOpen)
            {
                ////this sets prefix for reader to work with DataReceiveEvent
                byte[] data = HexStringToByteArray("01 3C 53 45 52 49 45 4E 3D 30 3E 3C 52 45 47 49 45 4E 3D 30 3E 3C 4E 4F 52 45 41 44 3D 31 3E 3C 50 52 45 43 49 44 3D 31 3E 3C 50 52 45 46 49 58 3D 5B 53 4F 48 5D 5B 53 54 58 5D 3E 3C 53 55 46 46 49 58 3D 5B 45 54 58 5D 5B 45 4F 54 5D 3E 3C 4D 41 47 31 45 4E 3D 31 3E 3C 4D 41 47 32 45 4E 3D 31 3E 3C 4D 41 47 33 45 4E 3D 31 3E 3C 50 44 46 34 31 37 3D 31 3E 3C 4D 41 47 52 41 57 3D 30 3E 3C 52 45 47 4B 45 59 3A 3E 3C 53 45 52 49 41 4C 3A 3E 04"); //  clears send s/n and r/n from stream

                // Send the binary data out the port
                comport.Write(data, 0, data.Length);
            }

        }

        #endregion

        private void btnInterface_Click(object sender, EventArgs e)
        {

            byte[] BarCode_Scan = HexStringToByteArray("40 0A 1C 0D 41 41 4D 56 41 33 36 30 30 31 30 30 31 44 4C 30 30 31 32 33 34 35 36 44 4C 0A 44 41 41 54 4F 4B 45 4E 57 4F 52 4B 53 40 54 4F 4D 40 54 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 0A 44 41 47 34 34 20 45 20 39 39 54 48 20 53 54 20 41 50 54 20 39 30 31 0A 44 41 49 4E 45 57 20 59 4F 52 4B 20 20 20 20 20 20 20 0A 44 41 4A 4E 59 0A 44 41 4B 31 30 31 32 38 20 20 20 20 20 20 0A 44 41 51 33 36 32 38 33 32 35 31 32 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 0A 20 44 42 42 31 39 38 37 30 38 30 32 0A 44 42 44 32 30 30 30 31 30 30 34 0A 44 42 41 32 30 31 33 30 35 30 37 0A 44 41 52 44 4D 20 20 0A 44 41 53 20 20 20 20 20 20 20 20 20 20 0A 44 41 54 20 20 20 20 20 20 0A 44 42 43 4D 0A 44 41 55 36 30 32 0A 44 41 59 42 52 20 0A 44 42 4B 20 20 20 20 20 20 20 20 20 0A 44 42 48 20 20 20 20 20 20 20 20 20 20 0D");
            Array.Resize(ref buff, BarCode_Scan.Length); //You MUST resize the data, the buffer sent must contain only card data, no preceding bytes generated by a scanner, and no trailing garbage data. Normally the length is created from the SerialPort.BytesToRead property.

            Array.Copy(BarCode_Scan, buff, BarCode_Scan.Length);// copy input to global buffer

            int i = -1;
            PdfLibTest.Barcode ibc;
            ibc = new PdfLibTest.Barcode();

            i = ibc.Parse417(buff); // Parse the buffer

            txtRetPDF.Text = ibc.problem_ID.ToString();
            txtTitle.Text = ibc.Title_ID;
            txtFirst.Text = ibc.FirstName_ID;
            txtMid.Text = ibc.Mid_ID;
            txtLast.Text = ibc.Last_ID;

            txtAddress.Text = ibc.Addr_ID;
            txtAddr2.Text = ibc.Addr2_ID;
            txtCity.Text = ibc.City_ID;
            txtState.Text = ibc.State_ID;
            txtZip.Text = ibc.Zip_ID;
            txtID.Text = ibc.Dl_ID;
            txtEXP.Text = ibc.Exp_ID;
            txtDobYear.Text = ibc.DobYear_ID;
            txtDobMMDD.Text = ibc.DobMMDD_ID;
            txtDBD.Text = ibc.DocIssDate_ID;

            txtHeight.Text = ibc.Height_ID;
            txtWeight.Text = ibc.Wt_ID;
            txtEye.Text = ibc.Eye_ID;
            txtSEX.Text = ibc.Sex_ID;
            txtHair.Text = ibc.Hair_ID;

            txtClass.Text = ibc.DLClass_ID;
            txtRestriction.Text = ibc.Rest_ID;
            txtEndorsement.Text = ibc.Endr_ID;

            txtCountry.Text = ibc.Country_ID;
            txtRace.Text = ibc.Race_ID;
           



        }

        private void btnIparsedMag_Click(object sender, EventArgs e)
        {
            byte[] Magstripe_Scan = HexStringToByteArray("25 4E 59 41 4C 42 41 4E 59 5E 54 4F 4B 45 4E 57 4F 52 4B 53 24 4A 4F 45 34 24 54 5E 38 33 34 20 43 4F 4E 43 48 20 53 48 45 4C 4C 20 52 44 20 20 20 20 23 39 30 31 5E 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 3F 3B 36 33 36 30 31 30 30 33 32 34 31 36 38 33 36 33 31 33 37 3D 30 37 30 32 31 39 38 34 39 39 30 34 30 3D 3F 25 21 20 31 32 33 34 34 30 30 30 30 20 20 45 20 42 20 20 20 20 20 20 20 20 20 20 20 20 20 31 35 30 36 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 20 4E 2F 26 2B 47 47 30 30 30 30 30 3F");
            Array.Resize(ref MagBuff, Magstripe_Scan.Length); //You MUST resize the data, the buffer sent must contain only card data, no preceding bytes generated by a scanner, and no trailing garbage data. Normally the length is created from the SerialPort.BytesToRead property.
            Array.Copy(Magstripe_Scan, 0, MagBuff, 0, Magstripe_Scan.Length); // copy the scan from your SerialDataReceivedEventHandler into the global MagBuff array

            int ret = -1;
            MagLibTest.Magstripe imc;
            imc = new MagLibTest.Magstripe();

            ret = imc.ParseDL(MagBuff); // Parse the buffer

            txtRetPDF.Text = imc.problem_ID.ToString();
            txtTitle.Text = imc.Title_ID;
            txtFirst.Text = imc.FirstName_ID;
            txtMid.Text = imc.Mid_ID;
            txtLast.Text = imc.Last_ID;

            txtAddress.Text = imc.Addr_ID;
            txtAddr2.Text = imc.Addr2_ID; //returns and empty string
            txtCity.Text = imc.City_ID;
            txtState.Text = imc.State_ID;
            txtZip.Text = imc.Zip_ID;
            txtID.Text = imc.Dl_ID;
            txtEXP.Text = imc.Exp_ID;
            txtDobYear.Text = imc.DobYear_ID;
            txtDobMMDD.Text = imc.DobMMDD_ID;
            txtDBD.Text = imc.DocIssDate_ID; //returns and empty string

            txtHeight.Text = imc.Height_ID;
            txtWeight.Text = imc.Wt_ID;
            txtEye.Text = imc.Eye_ID;
            txtSEX.Text = imc.Sex_ID;
            txtHair.Text = imc.Hair_ID;

            txtClass.Text = imc.DLClass_ID;
            txtRestriction.Text = imc.Rest_ID;
            txtEndorsement.Text = imc.Endr_ID;

            txtCountry.Text = imc.Country_ID; //returns and empty string
            txtRace.Text = imc.Race_ID; //returns and empty string


        }
    }
}